@@ -8,12 +8,11 @@
 # ***************************************************
 
 import io
-import os
 import time
+from pathlib import Path
 from copy import deepcopy
 from abc import ABC, abstractmethod
 from collections import OrderedDict
-from typing import Dict
 
 import numpy as np
 import torch
@@ -36,6 +35,28 @@ __all__ = [
 
 
 class BaseRunner(base.BaseEstimator, ABC):
+    """
+    PyTorchモデルの訓練に用いる Runner を作成する為のクラスです。
+
+
+    Examples:
+
+        >>> class Runner(BaseRunner):
+        >>>     def __init__(self):
+        >>>         super(Runner, self).__init__()
+        >>>         self.model = nn.Linear(10, 10)
+        >>>         self.optimizer = torch.optim.Adam(self.model.parameters())
+        >>>         self.experiment = Experiment()
+        >>>         self.criterion = nn.CrossEntropyLoss()
+        >>>
+        >>>     def train_step(self, batch):
+        >>>         x, y = batch
+        >>>         out = self.model(x)
+        >>>         loss = self.criterion(out, y)
+        >>>
+        >>>         return {"loss": loss}
+
+    """
     def __init__(self):
         self.device = torch.device("cuda" if torch.cuda.is_available() else "cpu")
         self.model = NotImplemented
@@ -48,6 +69,7 @@ class BaseRunner(base.BaseEstimator, ABC):
         self.pbar = None
         self._loaders = {}
         self._metrics = {}
+        self._checkpoint_path = None
 
     @abstractmethod
     def train_step(self, batch):
@@ -56,6 +78,7 @@ class BaseRunner(base.BaseEstimator, ABC):
             >>> for x, y in train_loader:
             >>>     out = model(x)
             >>>     loss = criteion(out, y)
+
         にあたる箇所を担当するメソッドです。
 
         Args:
@@ -70,6 +93,7 @@ class BaseRunner(base.BaseEstimator, ABC):
             >>>     out = self.model(x)
             >>>     loss = nn.functional.cross_entropy(out, y)
             >>>     return {"loss": loss}
+
         """
 
     def train_end(self, outputs):
@@ -82,19 +106,23 @@ class BaseRunner(base.BaseEstimator, ABC):
         Returns:
 
         """
+        return {}
 
     def val_step(self, batch):
         """
+        ニューラルネットの検証時、1step ごとに実行されるメソッドです。
 
         Args:
-            batch:
+            batch: PyTorch DataLoader から得られる訓練用のデータトラベルを含むタプル
 
         Returns:
+            辞書を返す必要があります。
 
         """
 
     def val_end(self, outputs):
         """
+        ニューラルネットの検証時、1step 終了ごとに実行されるメソッドです。
 
         Args:
             outputs:
@@ -102,19 +130,23 @@ class BaseRunner(base.BaseEstimator, ABC):
         Returns:
 
         """
+        return {}
 
     def test_step(self, batch):
         """
+        ニューラルネットの評価時、1step ごとに実行されるメソッドです。
 
         Args:
-            batch:
+            batch: PyTorch DataLoader から得られる訓練用のデータトラベルを含むタプル
 
         Returns:
+            辞書を返す必要があります。
 
         """
 
     def test_end(self, outputs):
         """
+        ニューラルネットの評価時、1step 終了ごとに実行されるメソッドです。
 
         Args:
             outputs:
@@ -122,8 +154,17 @@ class BaseRunner(base.BaseEstimator, ABC):
         Returns:
 
         """
+        return {}
 
     def train_cycle(self, epoch, loader):
+        """
+        ニューラルネットの訓練ループです。
+
+        Args:
+            epoch (int):
+            loader (torch.utils.data.DataLoader):
+
+        """
         results = list()
         loader_size = len(loader)
 
@@ -131,16 +172,17 @@ class BaseRunner(base.BaseEstimator, ABC):
         with self.experiment.train():
             for step, batch in enumerate(loader):
                 self.optimizer.zero_grad()
-                batch = tuple(map(lambda x: x.to(self.device), batch))
+                batch = modules.send(batch, self.device)
                 # on_step_start()
                 outputs = self.train_step(batch)
                 outputs["loss"].backward()
                 self.optimizer.step()
 
-                per = "{:1.0%}".format(step / loader_size)
-                self.pbar.set_postfix(
-                    OrderedDict(train_batch=per), refresh=True
-                )
+                if hasattr(self.pbar, "set_postfix"):
+                    per = "{:1.0%}".format(step / loader_size)
+                    self.pbar.set_postfix(
+                        OrderedDict(train_batch=per), refresh=True
+                    )
 
                 outputs = {
                     key: outputs[key].detach().cpu() if isinstance(outputs[key], torch.Tensor) else outputs[key]
@@ -151,10 +193,22 @@ class BaseRunner(base.BaseEstimator, ABC):
                 # on_step_end()
 
             dic = self.train_end(results)        # pylint: disable=E1111
-            self._metrics.update(dic)
-            self.experiment.log_metrics(dic, epoch=epoch)
+
+            if len(dic) != 0:
+                self._metrics.update(dic)
+                self.experiment.log_metrics(dic, step=epoch)
 
     def val_cycle(self, epoch, loader):
+        """
+        ニューラルネットの評価用ループです。
+
+        Args:
+            epoch:
+            loader:
+
+        Returns:
+
+        """
         results = list()
         loader_size = len(loader)
 
@@ -162,14 +216,15 @@ class BaseRunner(base.BaseEstimator, ABC):
         with self.experiment.validate():
             with torch.no_grad():
                 for step, batch in enumerate(loader):
-                    batch = tuple(map(lambda x: x.to(self.device), batch))
+                    batch = modules.send(batch, self.device)
                     # on_step_start()
                     outputs = self.val_step(batch)        # pylint: disable=E1111
 
-                    per = "{:1.0%}".format(step / loader_size)
-                    self.pbar.set_postfix(
-                        OrderedDict(val_batch=per), refresh=True
-                    )
+                    if hasattr(self.pbar, "set_postfix"):
+                        per = "{:1.0%}".format(step / loader_size)
+                        self.pbar.set_postfix(
+                            OrderedDict(val_batch=per), refresh=True
+                        )
 
                     outputs = {
                         key: outputs[key].cpu() if isinstance(outputs[key], torch.Tensor) else outputs[key]
@@ -180,10 +235,21 @@ class BaseRunner(base.BaseEstimator, ABC):
                     # on_step_end()
 
                 dic = self.val_end(results)        # pylint: disable=E1111
-                self._metrics.update(dic)
-                self.experiment.log_metrics(dic, epoch=epoch)
+
+                if len(dic) != 0:
+                    self._metrics.update(dic)
+                    self.experiment.log_metrics(dic, step=epoch)
 
     def test_cycle(self, loader):
+        """
+        ニューラルネットの検証用ループです。
+
+        Args:
+            loader:
+
+        Returns:
+
+        """
         results = list()
         loader_size = len(loader)
 
@@ -191,14 +257,16 @@ class BaseRunner(base.BaseEstimator, ABC):
         with self.experiment.test():
             with torch.no_grad():
                 for step, batch in enumerate(loader):
-                    batch = tuple(map(lambda x: x.to(self.device), batch))
+                    batch = modules.send(batch, self.device)
                     # on_step_start()
                     outputs = self.test_step(batch)        # pylint: disable=E1111
 
                     per = "{:1.0%}".format(step / loader_size)
-                    self.pbar.set_postfix(
-                        OrderedDict(test_batch=per), refresh=True
-                    )
+                    if hasattr(self.pbar, "set_postfix"):
+                        self.pbar.set_postfix(
+                            OrderedDict(test_batch=per), refresh=True
+                        )
+                        self.pbar.update(1)
 
                     outputs = {
                         key: outputs[key].cpu() if isinstance(outputs[key], torch.Tensor) else outputs[key]
@@ -210,15 +278,32 @@ class BaseRunner(base.BaseEstimator, ABC):
                     # on_step_end()
 
                 dic = self.test_end(results)        # pylint: disable=E1111
-                self._metrics.update(dic)
-                self.experiment.log_metrics(dic)
 
-    def train_config(self, epochs, *args, **kwargs):
+                if len(dic) != 0:
+                    self._metrics.update(dic)
+                    self.experiment.log_metrics(dic)
+
+    def train_config(self, epochs, **kwargs):
+        """
+        .run() メソッドを用いて訓練を行う際に、 epochs などを指定する為のメソッドです。
+
+        Args:
+            epochs (int):
+            **kwargs:
+
+        Returns:
+
+        """
+
+        self._checkpoint_path = kwargs.get("checkpoint_path", None)
+
         if epochs > 0:
             self._epochs = epochs
         else:
             self._epochs = 1
 
+        return self
+
     def log_hyperparams(self, dic=None, prefix=None):
         """
 
@@ -286,16 +371,20 @@ class BaseRunner(base.BaseEstimator, ABC):
                     # on_validation_end()
 
                 if self.scheduler:
-                    self.scheduler.step()
-                    self.experiment.log_metric("scheduler_lr", self.scheduler.get_last_lr(), epoch=epoch)
+                    self.scheduler.step(epoch=None)
+                    self.experiment.log_metric("scheduler_lr", self.scheduler.get_lr(), epoch=epoch)
 
                 if self.early_stop:
                     if self.early_stop.on_epoch_end(self._metrics, epoch):
                         break
                     # .on_epoch_end()
 
+                if self._checkpoint_path:
+                    self.save(self._checkpoint_path, epoch=epoch)
+
         if "test" in self.loaders:
             # on_test_start()
+            self.pbar = tqdm(total=len(self.loaders["test"]), desc="Evaluating") if verbose else None
             self.test_cycle(self.loaders["test"])
             # on_test_end()
 
@@ -303,6 +392,7 @@ class BaseRunner(base.BaseEstimator, ABC):
 
     def predict(self, x):
         """
+        与えられた入力をもとに予測を行うメソッドです。
 
         Args:
             x (Union[torch.Tensor, np.ndarray]):
@@ -310,16 +400,16 @@ class BaseRunner(base.BaseEstimator, ABC):
         Returns:
             predict
         """
+        raise NotImplementedError
 
     def add_loader(self, mode, loader):
         """
+        訓練等に用いるデータローダをRunnerに登録する為のメソッドです。
 
         Args:
-            mode (str):
+            mode (str): ['train', 'val', 'test'] のいずれをか指定します。
             loader (torch.utils.data.DataLoader):
 
-        Returns:
-
         """
         if mode not in ["train", "val", "test"]:
             raise Exception("argument `mode` must be one of 'train', 'val', or 'test'.")
@@ -338,12 +428,25 @@ class BaseRunner(base.BaseEstimator, ABC):
         return self._loaders
 
     def fit(self, x, y, **kwargs):
+        """
+        Scikit-Learn スタイルの訓練メソッドです。
+
+        Args:
+            x: 訓練用データ
+            y: 教師ラベル
+            **kwargs:
+
+        """
         val_size: float = kwargs.get("val_size", 0.1)
         num_workers = kwargs.get("num_workers", 0)
         batch_size = kwargs.get("batch_size", 1)
         pin_memory = kwargs.get("pin_memory", False)
         verbose = kwargs.get("verbose", True)
-        epochs = kwargs.get("epochs", 1)
+
+        if self._epochs == 0:
+            epochs = kwargs.get("epochs", 1)
+        else:
+            epochs = self._epochs
 
         train_ds = modules.get_dataset(x, y)
         val_ds = modules.get_dataset(x, y)
@@ -367,23 +470,41 @@ class BaseRunner(base.BaseEstimator, ABC):
 
         self.add_loader("train", train_loader)
         self.add_loader("val", val_loader)
-        self.train_config(epochs)
+        self.train_config(epochs, checkpoint_path=self._checkpoint_path)
         self.run(verbose)
+
         return self
 
     def freeze(self):
+        """
+        モデルのパラメータが勾配を計算しないように固定する為のメソッドです。
+
+        """
         for param in self.model.parameters():
             param.requires_grad = False
 
         self.model.eval()
 
     def unfreeze(self):
+        """
+        .freeze() で固定されたパラメータを再度学習できるようにする為のメソッドです。
+
+        """
         for param in self.model.parameters():
             param.requires_grad = True
 
         self.model.train()
 
-    def save_checkpoint(self) -> Dict[str, OrderedDict]:
+    def save_checkpoint(self):
+        """
+        ニューラルネットの重みと、 Optimizerの状態を辞書として出力するメソッドです。
+
+        Returns:
+            以下のキーと値を持つ辞書を返します。
+                - "model_state_dict": ニューラルネットの重み
+                - "optimizer_state_dict": Optimizer の状態
+
+        """
         if isinstance(self.model, nn.DataParallel):
             model = self.model.module.state_dict()
         else:
@@ -396,26 +517,45 @@ class BaseRunner(base.BaseEstimator, ABC):
         return checkpoint
 
     def load_checkpoint(self, checkpoint):
+        """
+        'model_state_dict', 'optimizer_state_dict' を持つ辞書を受け取り、それらを元に モデル と Optimizer の状態を復元します。
+
+        Args:
+            checkpoint:
+                以下のキーと値を持つ辞書。
+                    - "model_state_dict": ニューラルネットの重み
+                    - "optimizer_state_dict": Optimizer の状態
+        """
         self.model.load_state_dict(checkpoint["model_state_dict"])
         self.optimizer.load_state_dict(checkpoint["optimizer_state_dict"])
         return self
 
-    def save(self, directory, epoch=None) -> None:
+    def save(self, directory=None, epoch=None):
+
         """
+        指定したディレクトリにモデルとOptimizerの状態を記録したファイルを保存します。
+
         Args:
-            directory:
-            epoch:
-        Returns:
+            directory (Optional[str]):
+            epoch (Optional[int]):
+
         """
-        if not os.path.isdir(directory):
-            os.makedirs(directory)
+        if directory is None and self._checkpoint_path is not None:
+            directory = self._checkpoint_path
+
+        elif directory is None and self._checkpoint_path is None:
+            raise ValueError("The argument `directory` must be specified.")
+
+        directory = Path(directory)
+        if not directory.exists():
+            directory.mkdir(parents=True)
         checkpoint = self.save_checkpoint()
 
         if epoch is None:
             epoch = time.ctime().replace(" ", "_")
 
         filename = "checkpoint_epoch_{}.pth".format(epoch)
-        path = directory + filename
+        path = directory / filename
         torch.save(checkpoint, path)
 
         if hasattr(self.experiment, "log_asset_data"):
@@ -425,10 +565,12 @@ class BaseRunner(base.BaseEstimator, ABC):
 
     def load(self, filename, map_location="cpu"):
         """
+        指定したファイルを元にモデルとOptimizerの状態を復元します。
+
         Args:
-            filename:
-            map_location:
-        Returns:
+            filename (str):
+            map_location (str):
+
         """
         checkpoint = torch.load(filename, map_location=map_location)
         self.load_checkpoint(checkpoint)
